import { Component, OnInit , ChangeDetectionStrategy , ChangeDetectorRef , Renderer2 , ElementRef, ViewChild , inject } from '@angular/core';
import {FormControl, FormGroup, FormsModule, ReactiveFormsModule , Validators} from '@angular/forms';
import {MatIconModule} from '@angular/material/icon';
import {MatDividerModule} from '@angular/material/divider';
import {MatButtonModule} from '@angular/material/button';
import {provideNativeDateAdapter} from '@angular/material/core';
import {MatDatepickerModule} from '@angular/material/datepicker';
import {MatFormFieldModule} from '@angular/material/form-field';
import {MatInputModule} from '@angular/material/input';
import {MatSelectModule} from '@angular/material/select';
import {MatButtonToggleModule} from '@angular/material/button-toggle';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {ConformidadHelper} from "./helper/conformidad.helper";
import {NgClass, NgForOf} from "@angular/common";
import {MatCardModule} from '@angular/material/card';
import {Observable,from } from 'rxjs';
import {switchMap, map, startWith , debounceTime } from 'rxjs/operators';
import {AsyncPipe} from '@angular/common';
import {MatAutocompleteModule} from '@angular/material/autocomplete';
import {MatTooltipModule} from '@angular/material/tooltip';
import {MatDialog, MatDialogModule} from '@angular/material/dialog';
import {modalRequerimientoComponent} from "./modalRequerimiento/modalRequerimiento.componente";
import {MatTableModule} from '@angular/material/table';
import { MatCell, MatCellDef, MatColumnDef, MatHeaderCell, MatHeaderRow, MatHeaderRowDef, MatRow, MatRowDef, MatTable
} from "@angular/material/table";

interface tiposTrabajo {
  value: string;
  viewValue: string;
}

interface interRequerimientos {
  codigo: string;
  serie : string;
  numero: string;
}

interface interDetallePasaje {
  lugarOrigen: string;
  lugarDestino : string;
  monto: string;
  fecha: Date;
}

@Component({
  imports: [
    MatDatepickerModule,
    MatFormFieldModule,
    FormsModule,
    ReactiveFormsModule,
    MatSelectModule,
    MatInputModule,
    MatButtonToggleModule,
    MatCheckboxModule,
    NgForOf,
    MatCardModule,
    MatButtonModule,
    MatDividerModule,
    MatIconModule,
    MatAutocompleteModule,
    AsyncPipe,
    MatTooltipModule,
    NgClass,
    MatCell,
    MatCellDef,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderRow,
    MatHeaderRowDef,
    MatRow,
    MatRowDef,
    MatTable,
    MatTableModule,
  ],
  providers: [provideNativeDateAdapter()],
  selector: 'app-conformidad',
  standalone: true,
  templateUrl: './conformidad.componente.html',
  styleUrls: ['./conformidad.componente.css'],
  //changeDetection: ChangeDetectionStrategy.OnPush,
})

export class ConformidadComponent implements OnInit {


  // VIEWCHILD
  @ViewChild('labelAdvertencia', { static: false }) labelAdvertencia!: ElementRef;
  @ViewChild('inputBusqueda', { static: false }) inputBusqueda!: ElementRef;
  // VIEWCHILD


  //INICIALIZACION DE LISTADO DE TIPOS DE TRABAJO
  listTipoTrabajo: tiposTrabajo[] = [
    {value: '1', viewValue: 'Instalaciòn-Revisión'},
    {value: '2', viewValue: 'Instalaciòn-Ampliación'},
    {value: '3', viewValue: 'Mantenimiento-Reparación'},
    {value: '4', viewValue: 'Mantenimiento-Remodelación'},
    {value: '5', viewValue: 'Reposición'},
    {value: '6', viewValue: 'Reinstalación'},

  ];
  //INICIALIZACION DE LISTADO DE TIPOS DE TRABAJO

  // INICIALIZACION DEL RANGO DE FECHAS
  readonly range = new FormGroup({
    fechaInicio: new FormControl<Date | null>(null),
    fechaFin: new FormControl<Date | null>(null),
  })

  // INICIALIZACION DEL RANGO DE FECHAS

  // DATALIST

  personal_list: any = [];
  sede_list: any = [];
  filteredOptions: any[];
  cards:  { id: number; titulo: string; codigo: string}[] = [];
  idBotones: number = 0;
  labelText: string = ''; // Texto que se muestra inicialmente
  isFading: boolean = false; // Controla si la animación de desvanecimiento debe activarse
  selectedOption: string = '';  // Variable para enlazar con el input

  columnasPasaje: string[] = ['lugarOrigen', 'lugarDestino', 'monto', 'fecha','accion'];
  dataSource: interDetallePasaje[] = [];
  // DATALIST

  // GROUPO DE FORMULARIO


  datos_personal = new FormGroup({
     tecnicoPersonal : new FormControl(null ,Validators.required),
     sedePersonal : new FormControl(null,Validators.required),
     areaPersonal : new FormControl<String | null>(null ,Validators.required),
     pasajePersonal : new FormControl(null ,Validators.required),
  });

  datos_servicio = new FormGroup({
    requerimientoServicio : new FormControl(null ,Validators.required),
    descripcionServicio : new FormControl(null ,Validators.required),
    causaServicio : new FormControl(null ,Validators.required),
    selectedOption : new FormControl<String | null>(null ,[Validators.maxLength(12)]),
  });

  datos_trabajo = new FormGroup({
    tipoTrabajo : new FormControl(null ,Validators.required),
    descripcionTrabajo : new FormControl(null ,Validators.required),
  });

  datos_material = new FormGroup({
    materialSobrante : new FormControl(null ,Validators.required),
  });

  // GROUPO DE FORMULARIO

  constructor(
    private conformidadHelper: ConformidadHelper,
    private renderer: Renderer2,
    private detectorChage : ChangeDetectorRef,
    public dialog: MatDialog
  ) {
    this.filteredOptions = [];
    renderer.setStyle(document.body, 'background', 'var(--light-theme-bg)');
  }

  //ACCESOS RAPIDOS A GRUPOS DE FORMULARIOS

  accesoDP(){
    return this.datos_personal.controls;
  }

  accesoDS(){
    return this.datos_servicio.controls;
  }

  accesoDT(){
    return this.datos_trabajo.controls;
  }

  accesoDM(){
    return this.datos_material.controls;
  }

  //ACCESOS RAPIDOS A GRUPOS DE FORMULARIOS

  async ngOnInit(): Promise<void> {

    const [
      personal,
      sede
    ] = await Promise.all([
      this.conformidadHelper.cargarPersonal(),
      this.conformidadHelper.listadoGeneralHorizon({accion: 1}),
    ]);

    this.personal_list = personal;
    this.sede_list = sede;
    this.dataSource = [{ lugarOrigen: 'Lima', lugarDestino: 'Cusco', monto: 'S/. 100.00', fecha: new Date() }];
  }

  onEventSeleccion(event : any){
    const valor = event.option.value;

    if(this.cards.find((item) => item.codigo === valor)){
      this.labelText = 'Ya existe el requerimiento ' + valor + ' en el listado'; // Cambia el texto
      this.isFading = false; // Resetea el estado de opacidad para mostrar el texto

      // Después de 2 segundos, comienza a desvanecer el texto
      setTimeout(() => {
        this.isFading = true;
      }, 2000); // 2000 ms = 2 segundos

    }else{
      this.cards.push({ titulo: `Requerimiento ${valor}`, id: this.idBotones++ , codigo: valor });
      this.labelText = '';
    }
    this.datos_servicio.get('selectedOption')?.setValue('');
    this.inputBusqueda.nativeElement.blur();
    this.detectorChage.detectChanges();
    this.filteredOptions = [];

  }

  onEventTecnico(event: any) { // IMPRIMER EL ÁREA DEL TÉCNICO AL SELECCIONARLO

    const areaControl = this.accesoDP().areaPersonal;

    const codigoPer = event.value;
    const personal_list = this.personal_list;
    const personal_datos = personal_list.find((item: any) => item.codper === codigoPer);

    if(Object.keys(personal_datos).length  > 0){
      areaControl.setValue(personal_datos.descargo);
    }else{
      areaControl.setValue('Sin Cargo');
    }

    areaControl.updateValueAndValidity();
  }

  async onBusquedaRequerimiento(event: any) {
    const codigoSede = this.accesoDP().sedePersonal.value;
    const busquedaReq = event.target.value;

    const jsonParametrosReq = {
      accion : 2,
      nroRequerimiento : busquedaReq,
      sede : codigoSede
    };

    const requerimiento_list = await this.conformidadHelper.listadoGeneralHorizon(jsonParametrosReq) ;
    const mapeoResultado = requerimiento_list.map((item: any) => ({
      codigo: item.CODIGONUM,
      serie : item.CSERIE_DOC,
      numero: item.CODIGONUM
    }));

    this.filteredOptions = mapeoResultado;
  }

  deleteRequerimiento(valor : any){
    this.cards = this.cards.filter((item) => item.id !== valor);
  }

  async infoRequerimiento(valor: any){
    const requerimientoInfo = await this.conformidadHelper.listadoGeneralHorizon({accion: 3, nroRequerimiento: valor, sede: ''});

    const dialogRef = this.dialog.open(modalRequerimientoComponent,{
      width: '300rem',
      data: { requerimientoInfo }
    });
    dialogRef.afterClosed().subscribe(result => {
      console.log(`Dialog result: ${result}`);
    });
  }

}

